/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMMozActivityOptions;

[scriptable, uuid(e70c4181-ea3f-4aa5-a2f7-af910dc65e45)]
interface nsIDOMMozActivityRequestHandler : nsISupports
{
    void postResult(in jsval result);
    void postError(in DOMString error);
    readonly attribute nsIDOMMozActivityOptions source;
};
